/******************************************************************************************************************************
VERWENDUNG:	Prfgert
AUTHOR: 	  PHILIPP GAREIS
DATUM:		  14.01.2013
C:			    AVR
Compiler:	  GCC
VERSION:	  1.0.3
RELEASE:	  stable
*/

#include <avr/interrupt.h>								    //Interrupt Bibliothek
#include <avr/io.h>
#include "Globals.h"
#include "Timer.h"


/********** LOKALE DEFINITIONEN **********************************************************************************************/
volatile uint16_t systemtime = 0, systemtime_cmp = 0;
volatile uint16_t tmp_systemtime[16];					//8 verschiedene Timeouts mglich
volatile uint16_t tmp_rx_systemtime[16];      //16 Timeouts fr Ausgaberckstellung, falls auf dem RX CAN keine neuen Daten kommen


/********** TIMER0 INIT ******************************************************************************************************/
void init_Timer0(void)	//Millisekunden-Zhler
{
TCCR0A |= (1<<CS00) | (1<<CS01) | (1<<WGM01);	//Timer auf 1/64 des CPU Takts einstellen (250kHz oder 4s), Clear Timer on Compare match (CTC) mode
TIMSK0 |= (1<<OCIE0A);				                //Timer Interrupt aktivieren
OCR0A = 250;						                      //Compare match beim Erreichen von TCCR0 = OCR0 nach 1ms (4s * 250 = 1000s) --> Interrupt OCF0 wird ausgelst

// TCNT0 ^= aktueller Zhlwert des Counters TCCR0
}//end init_Timer0


/*********** TIMEOUT *********************************************************************************************************/
uint8_t Timeout (uint8_t timeout_array, uint8_t set_Timeout, uint16_t Timeout_time)
{
	volatile uint16_t systemtime_bak, diff_time;
	uint8_t n;
	
	for (n = 0; n < 5; n++)
	{
	  systemtime_bak = systemtime;                                            //Aktuelle Systemtime in lokale VAR speichern
	
	  if (systemtime_bak == systemtime_cmp)                                   //doppelter Vergleich, um Dateninkonsitenz zu vermeiden
	  {	
		  diff_time = systemtime_bak - tmp_systemtime[timeout_array];           //Differenz errechnen
		
	    if (set_Timeout == 1)
	    {
		    tmp_systemtime[timeout_array] = systemtime_bak;
	    }//end if 

	    if (diff_time >= Timeout_time)	                                      //An Funktion "1" zurckgeben, wenn Timeout erreicht wurde
	    {
		    return 1;
	    }//end if

	    else //(millisekunden < Timeout_erreicht)						                  //An Funktion "0" zurckgeben, wenn Timeout nicht erreicht wurde
	    {
		    return 0;
	    }//end else
	  }//end doppelter Vergleich, um Dateninkonsitenz zu vermeiden 	
	}//end 5 Versuche, um das Timeout auszuwerten
  return 2;                                                                 //Datenmll... 
}//end Timeout


/*********** TIMEOUT *********************************************************************************************************/
uint8_t rx_Timeout (uint8_t timeout_array, uint8_t set_Timeout, uint16_t Timeout_time)
{
	volatile uint16_t systemtime_bak, diff_time;
  uint8_t n;
	
	for (n = 0; n < 5; n++)
	{
	  systemtime_bak = systemtime;                                            //Aktuelle Systemtime in lokale VAR speichern
	
	  if (systemtime_bak == systemtime_cmp)                                   //doppelter Vergleich, um Dateninkonsitenz zu vermeiden
	  {	
		  diff_time = systemtime_bak - tmp_rx_systemtime[timeout_array];        //Differenz errechnen
		
	    if (set_Timeout == 1)
	    {
		    tmp_rx_systemtime[timeout_array] = systemtime_bak;
	    }//end if 

	    if (diff_time >= Timeout_time)	                                      //An Funktion "1" zurckgeben, wenn Timeout erreicht wurde
	    {
		    return 1;
	    }//end if

	    else //(millisekunden < Timeout_erreicht)						                  //An Funktion "0" zurckgeben, wenn Timeout nicht erreicht wurde
	    {
		    return 0;
	    }//end else
	  }//end doppelter Vergleich, um Dateninkonsitenz zu vermeiden	
  }//end 5 Versuche, um das Timeout auszuwerten	  
  return 2;                                                                 //Datenmll...
}//end Timeout


/*********** INTERRUPT TIMER0 ************************************************************************************************/
ISR (TIMER0_COMP_vect)		//IRQ so kurz wie mglich halten, um berschneidungen mit anderen IRQ's zu vermeiden (Flags setzten)
{
	systemtime++;			                                                        //Systemtime inkrementieren
  systemtime_cmp++;                                                         //zweite Systemtime inkrementieren
}//end TIMER0_COMP IRQ